@@ -5,19 +5,30 @@ from trytond.pool import Pool
 from .opportunity import *
 from .configuration import *
 from .sale import *
+from .party import *
+from .account import *
+from .product import *
+from .company import *
 
 
 def register():
     Pool.register(
         SaleOpportunity,
         SaleOpportunityLine,
-        SaleOpportunityHistory,
         SaleOpportunityEmployee,
         OpenSaleOpportunityEmployeeStart,
         SaleOpportunityMonthly,
         SaleOpportunityEmployeeMonthly,
         Configuration,
         Sale,
+        Party,
+        Address,
+        PaymentTerm,
+        PaymentTermLine,
+        Template,
+        Product,
+        Company,
+        Employee,
         module='sale_opportunity', type_='model')
     Pool.register(
         OpenSaleOpportunityEmployee,
