@@ -2,22 +2,21 @@
 # this repository contains the full copyright notices and license terms.
 "Sales extension for managing leads and opportunities"
 import datetime
-import time
 
-from sql import Column, Literal, Null
-from sql.aggregate import Min, Max, Count, Sum
-from sql.conditionals import Coalesce, Case
+from sql import Literal, Null
+from sql.aggregate import Max, Count, Sum
+from sql.conditionals import Case
 from sql.functions import Extract
 
 from trytond.model import ModelView, ModelSQL, Workflow, fields, Check
 from trytond.wizard import Wizard, StateView, StateAction, Button
 from trytond import backend
-from trytond.pyson import Equal, Eval, Not, In, If, Get, PYSONEncoder
+from trytond.pyson import Eval, In, If, Get, PYSONEncoder
 from trytond.transaction import Transaction
 from trytond.pool import Pool
 
 __all__ = ['SaleOpportunity', 'SaleOpportunityLine',
-    'SaleOpportunityHistory', 'SaleOpportunityEmployee',
+    'SaleOpportunityEmployee',
     'OpenSaleOpportunityEmployeeStart', 'OpenSaleOpportunityEmployee',
     'SaleOpportunityMonthly', 'SaleOpportunityEmployeeMonthly']
 
@@ -43,9 +42,9 @@ class SaleOpportunity(Workflow, ModelSQL, ModelView):
     'Sale Opportunity'
     __name__ = "sale.opportunity"
     _history = True
-    _rec_name = 'reference'
-    reference = fields.Char('Reference', readonly=True, required=True,
-        select=True)
+    _rec_name = 'number'
+    number = fields.Char('Number', readonly=True, required=True, select=True)
+    reference = fields.Char('Reference', select=True)
     party = fields.Many2One('party.party', 'Party', select=True,
         states={
             'readonly': Eval('state').in_(['converted', 'lost', 'cancelled']),
@@ -94,8 +93,6 @@ class SaleOpportunity(Workflow, ModelSQL, ModelView):
                 ['opportunity', 'lead', 'converted']),
             },
         depends=['state'], help="Percentage between 0 and 100")
-    history = fields.One2Many('sale.opportunity.history', 'opportunity',
-            'History', readonly=True)
     lost_reason = fields.Text('Reason for loss', states={
             'invisible': Eval('state') != 'lost',
             }, depends=['state'])
@@ -105,27 +102,31 @@ class SaleOpportunity(Workflow, ModelSQL, ModelView):
     def __register__(cls, module_name):
         pool = Pool()
         Sale = pool.get('sale.sale')
-        cursor = Transaction().cursor
+        cursor = Transaction().connection.cursor()
         TableHandler = backend.get('TableHandler')
         sql_table = cls.__table__()
         sale = Sale.__table__()
 
-        reference_exists = True
-        if TableHandler.table_exist(cursor, cls._table):
-            table = TableHandler(cursor, cls, module_name)
-            reference_exists = table.column_exist('reference')
-        super(SaleOpportunity, cls).__register__(module_name)
-        table = TableHandler(cursor, cls, module_name)
+        table = TableHandler(cls, module_name)
+        number_exists = table.column_exist('number')
 
-        # Migration from 2.8: make party not required and add reference as
+        # Migration from 3.8: rename reference into number
+        if table.column_exist('reference') and not number_exists:
+            table.column_rename('reference', 'number')
+            number_exists = True
+
+        super(SaleOpportunity, cls).__register__(module_name)
+        table = TableHandler(cls, module_name)
+
+        # Migration from 2.8: make party not required and add number as
         # required
         table.not_null_action('party', action='remove')
-        if not reference_exists:
+        if not number_exists:
             cursor.execute(*sql_table.update(
-                    columns=[sql_table.reference],
+                    columns=[sql_table.number],
                     values=[sql_table.id],
-                    where=sql_table.reference == Null))
-            table.not_null_action('reference', action='add')
+                    where=sql_table.number == Null))
+            table.not_null_action('number', action='add')
 
         # Migration from 3.4: replace sale by origin
         if table.column_exist('sale'):
@@ -231,11 +232,12 @@ class SaleOpportunity(Workflow, ModelSQL, ModelView):
         Sequence = pool.get('ir.sequence')
         Config = pool.get('sale.configuration')
 
+        config = Config(1)
         vlist = [x.copy() for x in vlist]
         for vals in vlist:
-            if not vals.get('reference'):
-                sequence = Config(1).sale_opportunity_sequence
-                vals['reference'] = Sequence.get_id(sequence.id)
+            if vals.get('number') is None:
+                vals['number'] = Sequence.get_id(
+                    config.sale_opportunity_sequence.id)
         return super(SaleOpportunity, cls).create(vlist)
 
     @classmethod
@@ -243,8 +245,7 @@ class SaleOpportunity(Workflow, ModelSQL, ModelView):
         if default is None:
             default = {}
         default = default.copy()
-        default.setdefault('reference', None)
-        default.setdefault('history', None)
+        default.setdefault('number', None)
         default.setdefault('sales', None)
         return super(SaleOpportunity, cls).copy(opportunities, default=default)
 
@@ -448,8 +449,7 @@ class SaleOpportunityLine(ModelSQL, ModelView):
     @classmethod
     def __register__(cls, module_name):
         TableHandler = backend.get('TableHandler')
-        cursor = Transaction().cursor
-        table = TableHandler(cursor, cls, module_name)
+        table = TableHandler(cls, module_name)
 
         super(SaleOpportunityLine, cls).__register__(module_name)
 
@@ -495,101 +495,6 @@ class SaleOpportunityLine(ModelSQL, ModelView):
         return sale_line
 
 
-class SaleOpportunityHistory(ModelSQL, ModelView):
-    'Sale Opportunity History'
-    __name__ = 'sale.opportunity.history'
-    date = fields.DateTime('Change Date')
-    opportunity = fields.Many2One('sale.opportunity', 'Sale Opportunity')
-    user = fields.Many2One('res.user', 'User')
-    party = fields.Many2One('party.party', 'Party', datetime_field='date')
-    address = fields.Many2One('party.address', 'Address',
-            datetime_field='date')
-    company = fields.Many2One('company.company', 'Company',
-            datetime_field='date')
-    employee = fields.Many2One('company.employee', 'Employee',
-            datetime_field='date')
-    start_date = fields.Date('Start Date')
-    end_date = fields.Date('End Date')
-    description = fields.Char('Description')
-    comment = fields.Text('Comment')
-    lines = fields.Function(fields.One2Many('sale.opportunity.line', None,
-            'Lines', datetime_field='date'), 'get_lines')
-    amount = fields.Numeric('Amount', digits=(16, Eval('currency_digits', 2)),
-        depends=['currency_digits'])
-    currency = fields.Function(fields.Many2One('currency.currency',
-            'Currency'), 'get_currency')
-    currency_digits = fields.Function(fields.Integer('Currency Digits'),
-        'get_currency_digits')
-    state = fields.Selection(STATES, 'State')
-    probability = fields.Integer('Conversion Probability')
-    lost_reason = fields.Text('Reason for loss', states={
-        'invisible': Not(Equal(Eval('state'), 'lost')),
-    }, depends=['state'])
-
-    @classmethod
-    def __setup__(cls):
-        super(SaleOpportunityHistory, cls).__setup__()
-        cls._order.insert(0, ('date', 'DESC'))
-
-    def get_currency(self, name):
-        return self.company.currency.id
-
-    def get_currency_digits(self, name):
-        return self.company.currency.digits
-
-    @classmethod
-    def table_query(cls):
-        Opportunity = Pool().get('sale.opportunity')
-        opportunity_history = Opportunity.__table_history__()
-        columns = [
-            Min(Column(opportunity_history, '__id')).as_('id'),
-            opportunity_history.id.as_('opportunity'),
-            Min(Coalesce(opportunity_history.write_date,
-                    opportunity_history.create_date)).as_('date'),
-            Coalesce(opportunity_history.write_uid,
-                opportunity_history.create_uid).as_('user'),
-            ]
-        group_by = [
-            opportunity_history.id,
-            Coalesce(opportunity_history.write_uid,
-                opportunity_history.create_uid),
-            ]
-        for name, field in cls._fields.iteritems():
-            if name in ('id', 'opportunity', 'date', 'user'):
-                continue
-            if hasattr(field, 'set'):
-                continue
-            column = Column(opportunity_history, name)
-            columns.append(column.as_(name))
-            group_by.append(column)
-
-        return opportunity_history.select(*columns, group_by=group_by)
-
-    def get_lines(self, name):
-        Line = Pool().get('sale.opportunity.line')
-        # We will always have only one id per call due to datetime_field
-        lines = Line.search([
-                ('opportunity', '=', self.opportunity.id),
-                ])
-        return [l.id for l in lines]
-
-    @classmethod
-    def read(cls, ids, fields_names=None):
-        res = super(SaleOpportunityHistory, cls).read(ids,
-            fields_names=fields_names)
-
-        # Remove microsecond from timestamp
-        for values in res:
-            if 'date' in values:
-                if isinstance(values['date'], basestring):
-                    values['date'] = datetime.datetime(
-                        *time.strptime(values['date'],
-                            '%Y-%m-%d %H:%M:%S.%f')[:6])
-                values['date'] = values['date'].replace(microsecond=0)
-                values['date'] += datetime.timedelta(seconds=1)
-        return res
-
-
 class SaleOpportunityReportMixin:
     number = fields.Integer('Number')
     converted = fields.Integer('Converted')
